///////////////////////////////////////////////////////////////////////////
// Spuzzum's Projectile Library - DEMO        by Jeremy "Spuzzum" Gibson //
///////////////////////////////////////////////////////////////////////////
//   This is just a demonstration of how to use it.  Run "s_missile.dmb" //
// to play.  Note that you can compile this at any time, in case the dmb //
// becomes obsolete or outdated.  If it becomes outdated to the point    //
// that it won't compile anymore, let me know and I'll fix it as soon as //
// I can.  Or, if you're feeling daring, you could always fix it yourself//
// and then send it to me =)                                             //
//                                                                       //
//   It's a little more complex than I planned originally, but, hey, at  //
// least it puts off a "wow" effect, which is always good. =)            //
///////////////////////////////////////////////////////////////////////////
// Objective:                                                            //
//   Simple.  Destroy all of the evil enemies that are invading your     //
// space.   Did I mention they were evil?  Oh, I did.                    //
//                                                                       //
// Controls:                                                             //
//   Click on a weapon from the control panel (the browse window).  This //
// will select this weapon, which can then be used against enemies.  Use //
// the arrow keys to move your fighter left and right, forward and back. //
//                                                                       //
// Weapons:                                                              //
//   Machine gun - click on a location, and you'll fire an endless flow  //
// of bullets at it.  Bullets must explode on top of an enemy ship to    //
// destroy it.                                                           //
//   Laser - click on a location and you'll fire a laser.  The laser     //
// destroys anything in its path.  The laser has 15 shots.               //
//   Cannon - click on a location and you'll fire a cannon shot at it.   //
// When it hits, it will destroy anything in a 1-turf radius.  This has  //
// 30 shots.                                                             //
//   Missile - click on a location to fire a missile at it.  If it hits  //
// anything, it will explode, taking out anything in a 1-turf radius.    //
// If it reaches its destination, it will explode there like a cannon.   //
// This has 5 shots.                                                     //
//   Smart bomb - click on this if you're in danger.  It'll blow up      //
// everything on the screen, no questions asked.  You only get one, so   //
// use it well.                                                          //
//                                                                       //
// Powerups:                                                             //
//   Ram these with your cruiser if you need something.                  //
//                                                                       //
//   Shields Up - will increase your deflector shield strength           //
//   Armor Up - will increase your hull strength                         //
//   Recharge Shields - will replenish a bit of your shields             //
//   Repair Armor - will repair a bit of your hull                       //
//                                                                       //
//   Laser Pack - will recharge 5 shots of your laser                    //
//   Cannon Ammo - will reload 5 shots of your cannon                    //
//   Missile - will reload 1 missile                                     //
//   Smart Bomb - will reload your smart bomb launcher                   //
//                                                                       //
// Good luck, captain!                                                   //
///////////////////////////////////////////////////////////////////////////

/*
Stars - s_missile('demo_star.dmi","medium_c",...)
 - fired at a target off screen, these give the effect of a starfield
Bullets - s_missile(/obj/projectile/bullet,...)
 - demonstrates the creation of new types
Missiles - s_missile(missile,...)
 - the lag effect shows how firing references can be useful
Cannon - s_missile('cannon.dmi',...)
 - because it takes a set amount of time to reach the destination,
   graphical effects can still have use
Laser - s_missile(/obj/projectile/beam,...)
 - shows fancier handling of new types, where you can do extraordinary
   things, like dense projectiles and the like!
*/

#define MG 1
#define CANNON 2
#define LASER 3
#define MISSILE 4

var
	mob/player

world
	name = "S_Missile Demo"
	mob = /mob/cruiser
	view = 5

turf
	icon = 'space.dmi'
	name = " "
	Click()
		switch(usr:weapon)
			if(MG)
				if(usr:target)
					usr:target.overlays -= 'crosshairs.dmi'
				src.overlays += 'crosshairs.dmi'
				usr:target = src
			if(CANNON)
				usr:Fire(src)
			if(LASER)
				usr:Fire(src)
			if(MISSILE)
				usr:Fire(src)

	barrier
		icon_state = "barrier"
		Enter(O)
			if(istype(O,/mob/cruiser))
				return 0
			return ..()

mob
	var
		move = 0
		delay = 10
		size = 5
		score = 0

	dead
		density = 0
		score = 0

	proc
		Explode()
			new /obj/blast(src.loc)
			del src

	Click()
		src.loc:Click()

	cruiser
		var
			shields = 20
			armor = 10
			maxshields = 20
			maxarmor = 10
			cannonammo = 30
			lasercharge = 15
			missiles = 5
			smartbomb = 1
			obj/statmeter/missile
			obj/statmeter/charge
			weapon = MG
			turf/target

		New()
			spawn() Tick()
			BulletFire()
			src.missile = new /obj/statmeter
			src.missile.icon = 'missiles.dmi'
			src.charge = new /obj/statmeter
			src.charge.icon = 'charge.dmi'

		proc
			BulletFire()
				if(src.target && src.weapon == MG)
					Fire(src.target)
				spawn(5) BulletFire()

			Fire(turf/T)
				switch(src.weapon)
					if(CANNON) //DEMONSTRATES GRAPHICAL-ONLY
						if(cannonammo == 0)
							usr << "Out of cannon ammo!"
							src.Machine_Gun()
							return
						usr:cannonammo -= 1
						s_missile('cannon.dmi',src.loc,T,3)
						spawn((get_dist(src.loc,T)*3)-1)
							var/mob/M
							for(M in range(1,T))
								if(!istype(M,/mob/cruiser))
									M:Explode()
								else
									damage(20)
							new /obj/blast(T)

					if(MISSILE) //DEMONSTRATES MOVING REFERENCES
						if(missiles == 0)
							usr << "Out of missiles!"
							src.Machine_Gun()
							return
						usr:missiles -= 1
						var/obj/projectile/firemissile/missile1 = new
						var/dir = pick(EAST,WEST)
						missile1.dir = dir
						missile1.loc = usr:loc
						spawn() missile1.Engage(T)
						T.overlays += 'crosshairs.dmi'
						spawn(10) T.overlays -= 'crosshairs.dmi'

					if(LASER) //DEMONSTRATES MOVING TYPES
						if(lasercharge == 0)
							usr << "Out of laser energy!"
							src.Machine_Gun()
							return
						usr:lasercharge -= 1
						s_missile(/obj/projectile/beam,src.loc,T)

					if(MG) //DEMONSTRATES RAPID-FIRE MOVING TYPES
						s_missile(/obj/projectile/bullet,src.loc,T)

			Tick()
				if(src.move)
					src.move--
				spawn(1) Tick()

			damage(num)
				if(num < shields)
					shields -= num
					return

				if(num > shields)
					num -= shields
					shields = 0

				if(num < armor)
					armor -= num
					return

				if(num > armor)
					usr << sound(null)
					usr << "GAME OVER!!"
					var/mob/dead/M = new
					M.key = usr.key
					M.loc = locate(/area/client_eye)
					M.score = src.score
					player = M
					M.move = 1
					del src

		icon = 'demo_player.dmi'

		Login()
			usr.loc = locate(/area/mob_login)
			usr.client.eye = locate(usr.x,usr.y+4,usr.z)
			usr << sound('axelay.mid',1)
			player = src
			..()

		Stat()
			statpanel("Status")
			stat("Cannon: ","[src.cannonammo]/30")
			src.missile.Update(src.missiles,5)
			stat("Missiles: ",src.missile)
			src.charge.Update(src.lasercharge,15)
			stat("Laser: ",src.charge)
			stat("Shields: ","[src.shields]/[src.maxshields]")
			stat("Armor: ","[src.armor]/[src.maxarmor]")
			stat("")
			stat("")
			stat("")
			stat("")
			stat("Score:",src.score)

		verb
			Machine_Gun()
				set category = "Console"
				src.weapon = MG
				if(src.target)
					src.target.overlays -= 'crosshairs.dmi'
					src.target = null
				usr << "Machine gun selected!"

			Laser()
				set category = "Console"
				src.weapon = LASER
				if(src.target)
					src.target.overlays -= 'crosshairs.dmi'
					src.target = null
				usr << "Laser selected!"

			Cannon()
				set category = "Console"
				if(src.target)
					src.target.overlays -= 'crosshairs.dmi'
					src.target = null
				src.weapon = CANNON
				usr << "Cannon selected!"

			Missile()
				set category = "Console"
				if(src.target)
					src.target.overlays -= 'crosshairs.dmi'
					src.target = null
				src.weapon = MISSILE
				usr << "Missile launcher selected!"

			Smart_Bomb()
				set category = "Console"
				if(smartbomb)
					var/mob/M
					for(M in range(10,usr))
						if(M!=usr)
							M.Explode()
					usr << "KABOOM!"
					src.smartbomb = 0
				else
					usr << "Smartbomb launcher is empty!"

		Bump(O)
			if(istype(O,/obj/powerup))
				switch(O:type)
					if(/obj/powerup/shieldcharge)
						if((maxshields-shields) > 5)
							shields += 5
							src << "Shields partially charged!"
						else
							shields = maxshields
							src << "Shields fully charged!"

					if(/obj/powerup/armorrepair)
						if((maxarmor-armor) > 5)
							shields += 5
							src << "Armor partially repaired!"
						else
							shields = maxshields
							src << "Armor fully repaired!"

					if(/obj/powerup/shieldboost)
						maxshields += 5
						shields += 5
						src << "Shield strength increased!"

					if(/obj/powerup/armorboost)
						maxarmor += 5
						armor += 5
						src << "Armor strength increased!"

					if(/obj/powerup/laserpack)
						if((15-lasercharge) > 5)
							lasercharge += 5
							src << "Lasers partially charged!"
						else
							lasercharge = 15
							src << "Lasers fully charged!"

					if(/obj/powerup/cannonammo)
						if((30-cannonammo) > 5)
							cannonammo += 5
							src << "Cannon partially reloaded!"
						else
							cannonammo = 30
							src << "Cannon fully reloaded!"

					if(/obj/powerup/missilepack)
						if((5-missiles) > 1)
							missiles += 1
							src << "Missile rack partially loaded!"
						else
							missiles = 5
							usr << "Missile rack fully loaded!"

					if(/obj/powerup/smartbomb)
						src << "Smartbomb ready!"
						smartbomb = 1

				del(O)
				return

			if(ismob(O))
				src.damage(O:size)
				if(src) del(O) //beats me, but it crashed so I tried this

	enemy
		proc/Process()
			spawn(rand(delay,shoot_speed)) Process()
		var/shoot_speed = 150
		New()
			..()
			spawn(rand(25,shoot_speed)) Process()
		Del()
			player.score += src.size
			..()

		Bump(mob/O)
			if(istype(O,/mob/cruiser))
				O.Bump(src)
			if(istype(O,/obj/star_target))
				player.score -= src.size
				del src

		shooter
			icon = 'demo_enemy.dmi'
			icon_state = "shooter"
			size = 10
			delay = 18
			Process()
				var/turf/target = ScatterShot(player.loc,1)
				s_missile(/obj/projectile/beam,src.loc,target)
				..()

		fighter
			icon = 'demo_enemy.dmi'
			icon_state = "fighter"
			size = 2
			delay = 20
			Process()
				var/turf/target = ScatterShot(player.loc,3)
				s_missile('cannon.dmi',src.loc,target,2)
				spawn(get_dist(src.loc,target)*2)
					var/mob/M
					for(M in range(1,target))
						if(!istype(M,/mob/cruiser))
							M:Explode()
						else
							M:damage(10)
				..()

		frigate
			icon = 'demo_enemy.dmi'
			icon_state = "frigate"
			size = 14
			delay = 8
			Process()
				var/turf/target = ScatterShot(player.loc,2)
				s_missile('cannon.dmi',src.loc,target,2)
				spawn(get_dist(src.loc,target)*2)
					var/mob/M
					for(M in range(1,target))
						if(!istype(M,/mob/cruiser))
							M:Explode()
						else
							M:damage(10)
				..()

		carrier
			icon = 'demo_enemy.dmi'
			icon_state = "carrier"
			size = 20
			delay = 6
			Process()
				var/mob/enemy/fighter/M = new
				M.loc = src.loc
				M.Move(get_step(M,pick(EAST,WEST,NORTH)))
				..()

		destroyer
			icon = 'demo_enemy.dmi'
			icon_state = "destroyer"
			size = 16
			delay = 10
			Process()
				var/turf/target = ScatterShot(player.loc,3)
				s_missile(/obj/projectile/beam,src.loc,target)
				..()

		tugboat
			icon = 'demo_enemy.dmi'
			icon_state = "tugboat"
			size = 12
			delay = 12
			Process()

		freighter
			icon = 'demo_enemy.dmi'
			icon_state = "freighter"
			size = 18
			delay = 15
			Process()

area
	mob_login
	client_eye

obj
	blast
		icon = 'blast.dmi'
		layer = 5
		New()
			..()
			spawn(5) del(src)

	projectile
		layer = 5
		firemissile
			density = 1
			dir = NORTH
			icon = 'missile.dmi'

			Bump(O)
				if(!istype(O,/mob/cruiser))
					O:Explode()
				del src

			proc/Engage(turf/T)
				sleep(2)
				src.Move(get_step(src,src.dir))
				sleep(2)
				src.Move(get_step(src,src.dir))
				sleep(2)
				src.Move(get_step(src,src.dir))
				sleep(2)
				src.icon_state = "moving"
				s_missile(src,src.loc,T,1,1)

			Del()
				new /obj/blast(src.loc)
				var/mob/M
				for(M in view(1,src))
					if(!istype(M,/mob/cruiser))
						M:Explode()
					else
						M:damage(25)
				..()

		beam
			icon = 'laser.dmi'
			density = 1
			Bump(O)
				src.loc = O:loc
				if(!istype(O,/mob/cruiser))
					O:Explode()
				else
					O:damage(15)

		bullet
			icon = 'cannon.dmi'

			Del()
				var/mob/M
				for(M in src.loc)
					if(!istype(M,/mob/cruiser))
						M:Explode()
					else
						M:damage(1)
				..()

	statmeter
		proc
			Update(N as num,O as num)
				src.icon_state = "[N]"
				src.name = " [N]/[O]"

	star_generator //GENERATES THE STARFIELD, ENEMIES, AND POWERUPS
		icon = 'demo_star.dmi'
		icon_state = "close_c"
		var/delay = 100

		New()
			spawn(rand(1,20)) Generate()
			spawn(rand(100,1200)) Goodies()
			spawn(rand(200,600)) Enemies()
			spawn(rand(200,1000)) Delay()
			..()

		proc
			Delay()
				delay--
				spawn(rand(100,500)) Delay()
			Enemies()
				var/mob/M
				switch(rand(1,9))
					if(1)
						M = new /mob/enemy/carrier
					if(2)
						M = new /mob/enemy/destroyer
					if(3)
						M = new /mob/enemy/fighter
					if(4)
						M = new /mob/enemy/freighter
					if(5)
						M = new /mob/enemy/frigate
					if(6)
						M = new /mob/enemy/shooter
					if(7)
						M = new /mob/enemy/tugboat

				if(M)
					M.loc = src.loc
					walk(M,SOUTH,M.delay)

				spawn(rand(delay,delay*3)) Enemies()

			Goodies()
				switch(rand(1,90))
					if(1 to 15)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/shieldcharge,src.loc,\
									star_target.loc,5)

					if(16 to 25)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/armorrepair,src.loc,\
									star_target.loc,5)

					if(26 to 30)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/shieldboost,src.loc,\
									star_target.loc,5)

					if(31 to 35)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/armorboost,src.loc,\
									star_target.loc,5)

					if(36 to 38)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/laserpack,src.loc,\
									star_target.loc,5)

					if(39 to 41)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/cannonammo,src.loc,\
									star_target.loc,5)

					if(42 to 44)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/missilepack,src.loc,\
									star_target.loc,5)

					if(45 to 47)
						//world << "Goody!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile(/obj/powerup/smartbomb,src.loc,\
									star_target.loc,5)

				spawn(rand(100,1200)) Goodies()

			Generate()
				switch(rand(1,6))
					if(1 to 3)
						//world << "Star!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile('demo_star.dmi',pick("distant_l","distant_c",\
									"distant_r"),src.loc,star_target.loc,3)

					if(4 to 5)
						//world << "Star!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile('demo_star.dmi',pick("medium_l","medium_c",\
									"medium_r"),src.loc,star_target.loc,2)

					if(6)
						//world << "Star!"
						var/obj/star_target/star_target
						for(star_target in world)
							if(star_target.x == src.x)
								s_missile('demo_star.dmi',pick("close_l","close_c",\
									"close_r"),src.loc,star_target,loc,1)

				spawn(rand(1,20)) Generate()

	star_target
		icon = 'demo_star.dmi'
		icon_state = "close_c"
		density = 1

	powerup
		Bump(mob/O)
			if(istype(O,/mob/cruiser))
				O.Bump(src)
			if(istype(O,/obj/star_target))
				del src
		density = 1
		icon = 'powerups.dmi'
		shieldcharge
			icon_state = "shieldcharge"
		shieldboost
			icon_state = "shieldboost"
		armorrepair
			icon_state = "armorrepair"
		armorboost
			icon_state = "armorboost"
		laserpack
			icon_state = "laserpack"
		cannonammo
			icon_state = "cannonammo"
		missilepack
			icon_state = "missile"
		smartbomb
			icon_state = "smartbomb"

client
	Northwest()
		return
	Northeast()
		return
	Southwest()
		return
	Southeast()
		return
	East()
		if(usr:move)
			return
		usr:move = 3
		..()
	South()
		if(usr:move)
			return
		usr:move = 3
		..()
	West()
		if(usr:move)
			return
		usr:move = 3
		..()
	North()
		if(usr:move)
			return
		usr:move = 3
		..()

proc
	ScatterShot(target,dist=1)
		if(!isturf(target))
			target = target:loc
		var/turf/T
		var/L[0]
		for(T in range(dist,target))
			L += T
		return L[rand(1,L.len)]